/* Copyright (c) 2011 Asustor Inc. All rights reserved. */

Ext.ns('AS.ARC.datasyncDropbox');


/**
 * @class AS.ARC.apps.datasyncDropbox.fileList
 * @extends AS.ARC.flexbox
 * @author JerryJiang 2018-11-28
 * If it found files cannot be synchronized, it will show and provide a link to list the files.
 */
Ext.define('AS.ARC.apps.datasyncDropbox.fileList', {
    extend: 'AS.ARC.flexbox',
    alias: ['widget.fileList'],

    API_URL: AS.ARC.util.getUserAppsPath() + 'datasyncDropbox/dropbox.cgi',

    hidden: true,

    initComponent: function() {
        var fileList = this;
        this.items = [{
            xtype: 'component',
            autoEl: {
                tag: 'div',
                cls: 'status-icon-16 warning',
                style: 'display: inline-block; margin: 0px 5px 0px 0px;'
            }
        }, {
            xtype: 'displayfield',
            itemId: 'link',
            value: Ext.String.format(_S('DATASYNC_DROPBOX', 'SOME_FILES_CANNOT_BE_SYNCHRONIZED'), '<a class="hyperlink-string" href="#">', '</a>'),
            listeners: {
                afterrender: function(me) {
                    me.el.dom.querySelector('a.hyperlink-string').addEventListener('click', function() {
                        fileList.openFileList();
                    });
                }
            }
        }];
        this.callParent(arguments);
    },

    openFileList: function() {
        var me = this;

        me.gridFileList = Ext.create('Ext.grid.GridPanel', {
            itemId: 'grid-file-list',
            height: 320,
            store: me.getStore([]),
            columns: [{
                text: _S('COMMON', 'NAME'),
                dataIndex: 'info',
                draggable: false,
                flex: 2.5,
                renderer: function(value, cell) {
                    cell.tdAttr = 'data-qtip="' + Ext.htmlEncode(Ext.htmlEncode(value)) + '"';
                    return Ext.String.htmlEncode(value.split('/').pop());
                }
            }, {
                text: _S('COMMON', 'SIZE'),
                dataIndex: 'size',
                draggable: false,
                flex: 1,
                renderer: function(value, cell, record) {
                    return AS.ARC.util.convertSizeUnit(value);
                }
            }, {
                text: _S('COMMON', 'STATUS'),
                dataIndex: 'type',
                draggable: false,
                flex: 3,
                renderer: function(value, cell, record) {
                    var s = value === 'Upload' ? _S('DATASYNC_DROPBOX', 'UPLOAD_FAILED') : _S('DATASYNC_DROPBOX', 'DOWNLOAD_FAILED');
                    var error = record.data.msg ? Ext.htmlEncode(record.data.msg) : _S('DATASYNC_DROPBOX', 'SYNCHRONIZATION_FAILED');

                    cell.style = 'display: flex';
                    return s + '<div class="status-icon-16 warning" style="margin-left: 5px;" data-errorqtip="' + Ext.htmlEncode(error) + '"></div>';
                }
            }]
        });

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('DATASYNC_DROPBOX', 'FILE_LIST'),
            width: 700,
            height: 450,
            parentWin: me.fn.win,
            items: [{
                xtype: 'asPanelTop10',
                items: [me.gridFileList]
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'CLOSE'),
                handler: function() {
                    win.close();
                }
            }],
            listeners: {
                afterrender: function() {
                    me.gridFileList.el.mask(_S('COMMON', 'LOADING'));
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(me.API_URL, {act: 'error_status'}),
                        method: 'post',
                        success: function (json) {
                            me.gridFileList.el.unmask();
                            me.gridFileList.store.loadData(json.error_status);
                        },
                        failure: function (json) {
                            AS.ARC.util.showMsgWindow({
                                5006: _S('DATASYNC_DROPBOX', 'UNKNOW_ERROR'),
                                5402: _S('DATASYNC_DROPBOX', 'OUT_OF_MEMORY'),
                            }, json, win);
                        }
                    });
                }
            }
        });

        win.show();
    },

    getStore: function(data) {
        return {
            store: 'array',
            fields: ['id', 'info', 'size', 'type', 'msg'],
            data: data
        };
    }
});

/**
 * @class AS.ARC.apps.datasyncDropbox.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.datasyncDropbox.core', {
    extend: 'Ext.util.Observable',

    apiUrl: AS.ARC.util.getUserAppsPath() + 'datasyncDropbox/dropbox.cgi',
    imageUrl: AS.ARC.util.getUserAppsPath() + 'datasyncDropbox/images/',

    constructor: function (config) {
        Ext.apply(this, config);
        this.callParent();
        this.init();
    },

    init: function () {
        var fn = this;
        fn.win = fn.desktop.createWindow({
            app: fn.app,
            id: fn.id,
            itemId: fn.id,
            width: 880,
            height: 520,
            resizable: true,
            border: false,
            layout: 'border',
            bodyStyle: 'padding: 0; border: none; background: #FFFFFF;',
            items: [
                fn.getMainPanel(),
                fn.getWestPanel()
            ],
            listeners: {
                afterrender: function (win) {
                    win.header.addCls('as-window-header-no-tab');
                    win.header.items.items[1].hide();
                    fn.win.el.mask(_S('COMMON','LOADING'));

                    //check cgi is working or not
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'check_login'}),
                        method: 'get',
                        success: function (json, opts) {
                            fn.mainPanel.loadData();
                        },
                        failure: function (json, opts) {
                            AS.ARC.util.showMsgWindow({}, json, fn.win);
                        }
                    });

                }
            }
        });
    },

    // To check if unlink or network disconnect
    connCounter: {
        times: 0,
        add: function () {
            this.times++;
        },
        get: function () {
            return this.times;
        },
        clear: function () {
            this.times = 0;
        }
    },
    //user info
    getStatus: function () {
        var fn = this;
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'status'}),
            method: 'post',
            success: function (json, opts) {
                fn.accountPanel.items.get('fs').items.get('status').setValue(fn.convertStatus(json.status));
                fn.status = setTimeout(function () {
                    fn.getStatus();
                }, 5000);

                fn.accountPanel.items.get('title').items.get('start').hide();
                fn.accountPanel.items.get('title').items.get('stop').show();

                fn.accountPanel.down('#error-file-list').setVisible(json.has_error);
            },
            failure: function (json, opts) {
                fn.accountPanel.items.get('fs').items.get('status').setValue(AS.ARC.datas.defaultValue);
                fn.status = setTimeout(function () {
                    fn.getStatus();
                }, 5000);

                fn.accountPanel.items.get('title').items.get('start').show();
                fn.accountPanel.items.get('title').items.get('stop').hide();
            }
        });
    },
    convertStatus: function(status) {
        var fn = this;
        var resTable = '';
        if(Array.isArray(status)) {
            resTable += '<table style="font-size:12px;width:370px;table-layout:fixed;">' +
                '<tr>' +
                    '<td style="width:15px"></td>' +
                    '<td style="width:210px"></td>' +
                    '<td style="width:150px"></td>' +
                '</tr>';
            Ext.each(status, function(fileStatus) {
                var description = '';
                var type = '';
                var icon = '';
                var fileName = '';
                if (fileStatus.progress) {
                    /*
                     *Download/Upload/Update/Verify ⇩/⇧/↺/⇅
                     *{0}:type
                     *{1}:icon
                     *{2}:file path
                     *{3}:file name
                     *{4}:byte rate
                     *{5}:elapsed
                     *{6}:elapsed time
                     *{7}:remaining
                     *{8}:remaining time
                     *{9}:progress
                     */
                    description = '<tr>' +
                        '<td style="width:15px"><abbr title="{0}"><b>{1}</b></abbr></td>' +
                        '<td style="width:210px;overflow:hidden;white-space:nowrap;text-overflow:ellipsis;padding-right:1px">' +
                            '<abbr title="{2}">{3}</abbr>' +
                        '</td>' +
                        '<td style="width:150px;">' +
                            '<abbr title="{4}/s\n{5}: {6}\n{7}: {8}">' +
                                '<progress max="100" value="{9}" style="width:140px"></progress>' +
                            '</abbr>' +
                        '</td>' +
                    '</tr>';
                    fileName = fileStatus.info.split('/')[fileStatus.info.split('/').length - 1];
                    if (fileStatus.type === 'Download') {
                        type = _S('APPCENTRAL', 'DOWNLOADING');
                        icon = '&#x21E9;';
                    }
                    if (fileStatus.type === 'Upload') {
                        type = _S('FILE_EXPLORER', 'WAIT_UPLOAD');
                        icon = '&#x21E7;';
                    }
                    if (fileStatus.type === 'Update') {
                        type = _S('COMMON', 'UPDATING');
                        icon = '&#x21BA;';
                    }
                    if (fileStatus.type === 'Verify') {
                        type = _S('COMMON', 'CHECKING');
                        icon = '&#x21C5;';
                    }
                    
                    resTable += Ext.String.format(description,
                                  type,
                                  icon,
                                  fileStatus.info,
                                  fileName,
                                  AS.ARC.util.convertSizeUnit(fileStatus.progress.byterate),
                                  _S('FTPBACKUP', 'ELAPSED_TIME'),
                                  fn.convertTimeFormat(fileStatus.progress.elapsed),
                                  _S('FTPBACKUP', 'REAMAINING_TIME'),
                                  fn.convertTimeFormat(fileStatus.progress.remain),
                                  fileStatus.progress.percentage
                                );
                } else if (fileStatus.info !== '') {
                    //CreateFolder/DeleteFile/DeleteFolder
                    /*
                     *{0}:type
                     *{1}:info
                     */
                    description = '<tr>' +
                        '<td colspan="2">{0}</td>' +
                        '<td style="width:150px">{1}</td>' +
                        '</tr>';
                    if (fileStatus.type === 'CreateFolder') {
                        type = _S('DATASYNC_DROPBOX', 'CREATE_FOLDER');
                    }
                    if (fileStatus.type === 'DeleteFile') {
                        type = _S('FILE_EXPLORER', 'TITLE_DELETE');
                    }
                    if (fileStatus.type === 'DeleteFolder') {
                        type = _S('COMMON', 'DELETE_FOLDER');
                    }
                    resTable += Ext.String.format(description, type, fileStatus.info);
                } else {
                    if (fileStatus.type === 'LocalNoSpec' || fileStatus.type === 'RemoteNoSpace' || fileStatus.type === 'NoFolder') {
                        /*
                         *{0}:type
                         */
                        description = '<tr>' +
                            '<td style="width:15px"><b>&#x2716;</b></td>' +
                            '<td style="width:210px">{0}</td>' +
                            '<td style="width:150px"></td>' +
                        '</tr>';
                    } else {
                        /*
                         *{0}:type
                         */
                        description = '<tr>' +
                            '<td colspan="2">{0}</td>' +
                            '<td style="width:150px"></td>' +
                        '</tr>';
                    }
                    
                    //LocalNoSpec/RemoteNoSpace/NoFolder
                    if (fileStatus.type === 'LocalNoSpec') {
                        
                        type = _S('DATASYNC_DROPBOX', 'LOCAL_FULL');
                    }
                    if (fileStatus.type === 'RemoteNoSpace') {
                        type = _S('DATASYNC_DROPBOX', 'REMOTE_FULL');
                    }
                    if (fileStatus.type === 'NoFolder') {
                        type = _S('DATASYNC_DROPBOX', 'NOT_EXISTED_FOLDER');
                    }
                    //Idle/Init/RemoteIndex/LocalIndex/Check
                    if (fileStatus.type === 'Idle') {
                        type = _S('DATASYNC_DROPBOX', 'IDLE');
                    }
                    if (fileStatus.type === 'Init') {
                        type = _S('COMMON', 'INITIALIZING');
                    }
                    if (fileStatus.type === 'RemoteIndex') {
                        type = _S('DATASYNC_DROPBOX', 'REMOTE_INDEX');
                    }
                    if (fileStatus.type === 'LocalIndex') {
                        type = _S('DATASYNC_DROPBOX', 'LOCAL_INDEX');
                    }
                    if (fileStatus.type === 'Check') {
                        type = _S('COMMON', 'CHECKING');
                    }
                    resTable += Ext.String.format(description, type);
                }
                
            });
            resTable += '</table>';
        }
        //console.log(resTable);
        return resTable;
    },
    convertTimeFormat: function(value) {
        var res = '';
        var hours = 0;
        var minutes = 0;
        var seconds = 0;
        var zeroFill = function(number, width) {
            width -= number.toString().length;
            if ( width > 0 ) {
                return new Array( width + (/\./.test( number ) ? 2 : 1) ).join( '0' ) + number;
            }
            return number + "";
        }
        if(isNaN(value)) {
            return '--:--:--';
        }
        hours = zeroFill(Math.floor(value / (60 * 60)), 2);
        minutes = zeroFill(Math.floor((value % (60 * 60)) / 60), 2);
        seconds = zeroFill(Math.floor((value % (60 * 60)) % 60), 2);

        res = hours + ':' + minutes + ':' + seconds;
        return res;
    },
    getUserInfo: function () {
        var fn = this;

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'userinfo'}),
            method: 'post',
            success: function (json, opts) {
                
                var quota = json.userinfo.quota;
                var availableQuota = json.userinfo.available;
                var quotaGB = AS.ARC.util.convertSizeUnit(quota);
                var availableGB = AS.ARC.util.convertSizeUnit((availableQuota < 0) ? 0 : availableQuota);
                var usedGB = AS.ARC.util.convertSizeUnit(quota - availableQuota);
                var progValue = (quota - availableQuota) / quota;
                var progText = '';
                var capacityBar = fn.accountPanel.items.get('fs').items.get('capacityBar');
                var sizeBar = fn.accountPanel.items.get('fs').items.get('size-bar');

                if (progValue > 0) {
                    progText = (progValue * 100).toFixed(2) + '%';
                } else {
                    progText = '0 %';
                }

                capacityBar.updateProgress(Math.min(Math.max(0, progValue), 1), progText);
                sizeBar.items.get('used').setValue(_S('COMMON', 'USED') + ' ' + usedGB);
                sizeBar.items.get('free').setValue(_S('COMMON', 'FREE') + ' ' + availableGB);
                fn.accountPanel.items.get('fs').items.get('name').setValue(json.userinfo['name']);
                fn.accountPanel.items.get('fs').items.get('link').setValue('<a href="' + json.userinfo.link + '" target="_blank">' + 'https://www.dropbox.com/' + '</a>');
                fn.accountPanel.items.get('fs').items.get('quota').setValue(quotaGB);

                fn.userInfo = setTimeout(function() {
                    fn.getUserInfo();
                }, 5000);
            },
            failure: function (json, opts) {
                fn.userInfo = setTimeout(function () {
                    fn.getUserInfo();
                }, 5000);
            }
        });
    },
    //layout
    getWestPanel: function () {
        var fn = this;

        fn.westPanel = Ext.create('AS.ARC.fnFormPanel', {
            cls: 'as-page-panel',
            page: fn,
            region: 'west',
            width: 245,
            height: 500,
            bodyStyle: 'border:none;background:transparent;',
            style: 'background-image: url("' + fn.imageUrl + 'left_border.png");'
                + 'background-repeat: no-repeat;'
                + 'border:none;'
                + 'background-position-y: 100%;'
        });

        return fn.westPanel;
    },
    getUploadNote: function () {
        return [
            {
                xtype: 'displayfield',
                cls: 'note-string',
                style: {
                    marginBottom: '0px',
                    paddingLeft: '10px'
                },
                value: _S('DATASYNC_DROPBOX', 'DATASYNC_DROPBOX_LIMIT_TEXT')
            },
            {
                xtype: 'displayfield',
                cls: 'note-string',
                style: {
                    marginBottom: '0px',
                    paddingLeft: '10px'
                },
                value: _S('DATASYNC_DROPBOX', 'DATASYNC_DROPBOX_LIMIT_RULE1')
            },
            {
                xtype: 'displayfield',
                cls: 'note-string',
                style: {
                    paddingLeft: '10px'
                },
                value: _S('DATASYNC_DROPBOX', 'DATASYNC_DROPBOX_LIMIT_RULE2')
            }
        ];
    },
    getLoginPanel: function () {
        var fn = this;

        var signBtn = Ext.create(Ext.button.Button, {
            text: _S('APPCENTRAL', 'SIGN_IN'),
            disabled: true,
            itemId: 'signinaddress',
            handler: fn.doLogin,
            scope: fn
        });

        fn.loginPanel = Ext.create('Ext.container.Container', {
            style: 'padding-top: 80px;text-align: center',
            border: false,
            items: [
                {
                    xtype: 'container',
                    height: 150,
                    width: '100%',
                    style: 'background-image: url("' + fn.imageUrl + 'signin.png");' +
                        'background-repeat: no-repeat;' +
                        'background-position: center center;'
                },
                signBtn
            ]
        });

        return fn.loginPanel;
    },
    getSettingPanel: function () {
        var fn = this;
        var finishValidate = function() {
            var root = fn.settingPanel.items.get('serverTreePanel').items.get('fs').items.get('servertree').getRootNode();
            var sizeLimit =  fn.settingPanel.items.get('filter-panel').items.get('size_limit');
            var uploadLimit = fn.settingPanel.items.get('configPanel').items.get('fs1').items.get('upload_limit');
            var downloadLimit = fn.settingPanel.items.get('configPanel').items.get('fs1').items.get('download_limit');
            //check serverTree is at last one checked folder
            var hasInvalid = function() {
                if (!root.findChild('checked', true, true)) {
                    return true;
                }
                if (!sizeLimit.isValid()) {
                    return true;
                }
                if (!uploadLimit.isValid()) {
                    return true;
                }
                if (!downloadLimit.isValid()) {
                    return true;
                }
                    return false;
            }
            if (nextBtn.getText() === _S('COMMON', 'FINISH')) {
                nextBtn.setDisabled(hasInvalid());
            }
        }
        //buttons
        var navHandler = function(direction) {
            var settingPanel = fn.settingPanel;
            var prev = prevBtn;
            var next = nextBtn;
            var activeIndex = settingPanel.items.indexOf(settingPanel.getLayout().activeItem);

            if(activeIndex == 0) {
                if(direction == 1) {
                    settingPanel.getLayout().setActiveItem(1);
                    prev.enable();
                }
            }  else if(activeIndex == 1) {
                if(direction == -1) {
                    settingPanel.getLayout().setActiveItem(0);
                    prev.disable();
                } else {
                    settingPanel.getLayout().setActiveItem(2);
                    next.setText(_S('COMMON', 'FINISH'));
                    prev.enable();
                    finishValidate();
                }
            } else if(activeIndex == 2) {
                if(direction == -1) {
                    settingPanel.getLayout().setActiveItem(1);
                    next.enable();
                    next.setText(_S('COMMON', 'NEXT'));
                } else {
                    fn.win.el.mask(_S('COMMON','LOADING'));
                    //folder setting
                    var path = settingPanel.items.get('serverTreePanel').items.get('path-ct').down('[itemId=path_df]').getValue();
                    var physicalPath = '\/volume1' + path;
                    var serverTree = settingPanel.items.get('serverTreePanel').items.get('fs').items.get('servertree');
                    var filterPath = [];
                    serverTree.getRootNode().cascadeBy(function(n) {
                        if(!n.data.root && !n.data.checked) {
                            filterPath.push(n.data.path);
                        }
                    });
                    //filter setting
                    var filterPanel = settingPanel.items.get('filter-panel');
                    var filterSetPanel = filterPanel.items.get('filter-set').items.get('filter-set-panel');
                    var documentCt = filterSetPanel.items.get('documentCt').items.get('ct2').query('checkbox');
                    var videoCt = filterSetPanel.items.get('videoCt').items.get('ct2').query('checkbox');
                    var musicCt = filterSetPanel.items.get('musicCt').items.get('ct2').query('checkbox');
                    var photoCt = filterSetPanel.items.get('photoCt').items.get('ct2').query('checkbox');
                    var applicationCt = filterSetPanel.items.get('applicationCt').items.get('ct2').query('checkbox');
                    var customCt = filterSetPanel.items.get('customCt').items.get('ct2').query('checkbox');
                    var sizeLimit = filterPanel.items.get('size_limit').getValue();
                    var filterExtName = [];
                    var filterName = [];
                    var userDefExtName = [];
                    for (var i = 0; i < documentCt.length; i ++) {
                        if (documentCt[i].getValue() == true) {
                            filterExtName.push(documentCt[i].boxLabel);
                        }
                    }
                    for (var i = 0; i < videoCt.length; i ++) {
                        if (videoCt[i].getValue() == true) {
                            filterExtName.push(videoCt[i].boxLabel);
                        }
                    }
                    for (var i = 0; i < musicCt.length; i ++) {
                        if (musicCt[i].getValue() == true) {
                            filterExtName.push(musicCt[i].boxLabel);
                        }
                    }
                    for (var i = 0; i < photoCt.length; i ++) {
                        if (photoCt[i].getValue() == true) {
                            filterExtName.push(photoCt[i].boxLabel);
                        }
                    }
                    for (var i = 0; i < applicationCt.length; i ++) {
                        if (applicationCt[i].getValue() == true) {
                            filterExtName.push(applicationCt[i].boxLabel);
                        }
                    }
                    for (var i = 0; i < customCt.length; i ++) {
                        userDefExtName.push(customCt[i].boxLabel);
                        if (customCt[i].getValue() == true) {
                            filterExtName.push(customCt[i].boxLabel);
                        }
                    }
                    //config setting
                    var configPanel = settingPanel.items.get('configPanel');
                        var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit').getValue();
                        var downloadLimit = configPanel.items.get('fs1').items.get('download_limit').getValue();
                        var radiogroup = configPanel.items.get('fs2').query('radiogroup')[0];
                        var enableUpload = true;
                        var enableDownload = true;
                        var deleteExtra = true;
                    if(radiogroup.getValue().syncOption == 'DROPBOX_TO_NAS') {
                        enableUpload = false;
                        deleteExtra = configPanel.items.get('fs2').items.get('delete_extra').getValue();
                    } else if(radiogroup.getValue().syncOption == 'NAS_TO_DROPBOX') {
                        enableDownload = false;
                        deleteExtra = configPanel.items.get('fs2').items.get('delete_extra').getValue();
                    }
                    var conf = {
                        'local_root': path,
                        'local_root_path': physicalPath,
                        'size_limit': sizeLimit,
                        'upload_limit': uploadLimit,
                        'download_limit': downloadLimit,
                        'upload': enableUpload,
                        'download': enableDownload,
                        'delete_remote': deleteExtra,
                        'delete_local': deleteExtra,
                        'filter_extname': filterExtName,
                        'user_def_extname': userDefExtName,
                        'filter_name': filterName,
                        'filter_path': filterPath
                    };
                    
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'set_conf', conf: JSON.stringify(conf)}),
                        method: 'post',
                        success: function (json, opts) {
                            fn.win.el.unmask();
                            fn.accountPanel.loadData();
                            fn.mainPanel.getLayout().setActiveItem(2);
                            var location = fn.accountPanel.items.get('fs').items.get('locationData').items.get('location');
                            location.setValue(path);
                        },
                        failure: function (json, opts) {
                            fn.win.el.unmask();
                        }
                    });
                }
            }
        }
        var prevBtn = Ext.create('Ext.button.Button', {
            xtype: 'button',
            itemId: 'prev',
            text: _S('COMMON', 'PREVIOUS'),
            handler: Ext.bind(navHandler, fn, [-1]),
            disabled: true
        });
        var nextBtn = Ext.create('Ext.button.Button', {
            xtype: 'button',
            itemId: 'next',
            text: _S('COMMON', 'NEXT'),
            handler: Ext.bind(navHandler, fn, [1])
        });
        
        var getFileBrowse = function() {
            var savePath = Ext.create('Ext.form.field.Text', {
                itemId: 'path_df',
                dirtyPath: false,
                width: 480,
                padding: 0,
                value: '',
                readOnly: true,
                allowBlank: false,
                blankText: _S('MESSAGE', 'HF00011'),
                listeners: {
                    afterrender: function(df) {
                        df.validate();
                        AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get_conf'}),
                            method: 'post',
                            success: function (json, opts) {
                                df.setValue(json.conf.local_root);
                            },
                            failure: function(json, opts) {
                            }
                        });
                    }
                }
            });
            var browseBtn = Ext.create(Ext.button.Button, {
                itemId: 'browse',
                margin: '0 0 0 10',
                text: _S('COMMON', 'BROWSE'),
                handler: function () {
                    var fs = Ext.create('AS.ARC.treeSelector', {
                        hideReadOnly: true,
                        parentWin: fn.win,
                        objectPath: savePath,
                        isShowHome: true,
                        hideUserName: true,
                        unselectableFolder: [
                            '/home/' + AS.ARC.user.account,
                            '/home/' + AS.ARC.user.account + '/.datasync-dropbox'
                        ]
                    });
                    fs.win.show();
                }
            });
            var ct = Ext.create(Ext.form.FieldSet, {
                itemId: 'path-ct',
                title: _S('DATASYNC_DROPBOX', 'PLEASE_SELECT_LOCAL_FOLDER'),
                // layout: 'hbox',
                items: [{
                    xtype: 'container',
                    layout: 'hbox',
                    margin: 0,
                    items: [
                        savePath,
                        browseBtn
                    ]
                }]
            });

            return ct;
        };
        var serverTree = fn.getServerTree();
        serverTree.on('itemclick', finishValidate);
        var serverTreePanel = Ext.create('Ext.panel.Panel', {
            itemId: 'serverTreePanel',
            //width: 600,
            height: 420,
            border: false,
            items: [
                getFileBrowse(),
                //folders in dropbox
                {
                    xtype: 'fieldset',
                    itemId: 'fs',
                    title: _S('DATASYNC_DROPBOX', 'DO_NOT_SYNC_FOLDER'),
                    border: false,
                    height: 300,
                    items: [
                        serverTree
                    ]
                },
                {
                    xtype: 'displayfield',
                    itemId: 'path',
                    hidden: true
                }
            ]
        });
        var filterPanel = fn.getFilterPanel();
        var sizeLimit = filterPanel.items.get('size_limit');
        sizeLimit.on('validchange', finishValidate);
        
        var configPanel = fn.getConfigPanel();
        var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit');
        uploadLimit.on('validitychange', finishValidate);
        var downloadLimit = configPanel.items.get('fs1').items.get('download_limit');
        downloadLimit.on('validitychange', finishValidate);
        
        var resetSettingPanel = function () {
            //filter setting
            var filterPanel = fn.settingPanel.items.get('filter-panel');
            var filterSetPanel = filterPanel.items.get('filter-set').items.get('filter-set-panel');
            var documentCt = filterSetPanel.items.get('documentCt').items.get('ct2').query('checkbox');
            var videoCt = filterSetPanel.items.get('videoCt').items.get('ct2').query('checkbox');
            var musicCt = filterSetPanel.items.get('musicCt').items.get('ct2').query('checkbox');
            var photoCt = filterSetPanel.items.get('photoCt').items.get('ct2').query('checkbox');
            var applicationCt = filterSetPanel.items.get('applicationCt').items.get('ct2').query('checkbox');
            var customCt = filterSetPanel.items.get('customCt').items.get('ct2');
            var sizeLimit = filterPanel.items.get('size_limit');
            //folder setting
            var path = fn.settingPanel.items.get('serverTreePanel').items.get('path-ct').down('[itemId=path_df]');
            var serverTree = fn.settingPanel.items.get('serverTreePanel').items.get('fs').items.get('servertree');
            //config setting
            var configPanel = fn.settingPanel.items.get('configPanel');
            var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit');
            var downloadLimit = configPanel.items.get('fs1').items.get('download_limit');
            var radiogroup = configPanel.items.get('fs2').query('radiogroup')[0];
            var deleteExtra = configPanel.items.get('fs2').items.get('delete_extra');

            customCt.removeAll();
            filterPanel.items.get('filter-set').checkSelectAll(filterSetPanel.items.get('customCt'));

            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get_conf'}),
                method: 'post',
                success: function (json, opts) {
                    path.setValue(json.conf.local_root);
                },
                failure: function(json, opts) {
                }
            });

            var root = serverTree.getStore().getProxy();
            root.extraParams.id = '';
            root.extraParams.path = '';
            if (!serverTree.getStore().isLoading()) {
                serverTree.el.mask();
                setTimeout(function() {
                    serverTree.el.mask(_S('COMMON', 'LOADING'));
                    serverTree.getStore().load();
                }, 100);
            }
            sizeLimit.setValue(0);
            uploadLimit.setValue(0);
            downloadLimit.setValue(0);
            radiogroup.setValue({syncOption: 'BOTH'});
            deleteExtra.setValue(true);


            for (var i = 0; i < documentCt.length; i ++) {
                documentCt[i].setValue(false);
            }

            for (var i = 0; i < videoCt.length; i ++) {
                videoCt[i].setValue(false);
            }

            for (var i = 0; i < musicCt.length; i ++) {
                musicCt[i].setValue(false);
            }

            for (var i = 0; i < photoCt.length; i ++) {
                photoCt[i].setValue(false);
            }

            for (var i = 0; i < applicationCt.length; i ++) {
                applicationCt[i].setValue(false);
            }

            customCt.removeAll();
        }
        
        fn.settingPanel = Ext.create('Ext.panel.Panel', {
            height: 530,
            activeItem: 0,
            layout: 'card',
            items: [
                serverTreePanel,
                filterPanel,
                configPanel
            ],
            dockedItems: [{
                xtype: 'toolbar',
                dock: 'bottom',
                ui: 'footer',
                height: 60,
                items: [
                    prevBtn,
                    nextBtn
                ]
            }],
            loadData: function() {
                fn.settingPanel.getLayout().setActiveItem(0);
                resetSettingPanel();
                prevBtn.disable();
                nextBtn.enable();
                nextBtn.setText(_S('COMMON', 'NEXT'));
            }
        });

        return fn.settingPanel;
    },
    getAccountPanel: function () {
        var fn = this;

        var capacityBar = Ext.create(Ext.ProgressBar, {
            itemId: 'capacityBar',
            margin: '15 0 0 0',
            cls: 'progressbar-size',
            listeners: {
                update: function(pb, value) {
                    var spaceError = AS.ARC.util.fixDc('apps/storageManager/images/space_used_error.png');
                    var spaceUsed = AS.ARC.util.fixDc('apps/storageManager/images/space_used.png');
                    var sizeBarIcon = sizeBar.items.get(0);
                    if(value >= 1) {
                        pb.addCls('x-progress-error');
                        sizeBarIcon.update("<div class='used-icon' style='background-color: #ff9d7a; margin-left: 100px;'></div>");
                    } else {
                        pb.removeCls('x-progress-error');
                        sizeBarIcon.update("<div class='used-icon' style='margin-left: 100px;'></div>");
                    }
                }
            }
        });

        var sizeBar = Ext.create(Ext.container.Container, {
            itemId: 'size-bar',
            layout: 'hbox',
            height: 30,
            items: [
            {
                xtype: 'component',
                html: "<div class='used-icon' style='margin-left: 100px;'></div>"
            },
            {
                xtype: 'displayfield',
                itemId: 'used',
                width: 180,
                style: 'margin-left: 20px; margin-top: 5px;'
            },
            {
                xtype: 'component',
                html: "<div class='free-icon'></div>"
            },
            {
                xtype: 'displayfield',
                itemId: 'free',
                width: 180,
                style: 'margin-left: 20px; margin-top: 5px;'
            }]
        });

        fn.accountPanel = Ext.create('Ext.container.Container', {
            cls: 'as-page-panel',
            height: 500,
            width: 845,
            border: false,
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthSmall
            },
            items: [{
                xtype: 'container',
                height: 30,
                itemId: 'title',
                layout: {
                    type: 'hbox'
                },
                items: [{
                    xtype: 'button',
                    itemId: 'start',
                    text: _S('DATASYNC_DROPBOX', 'START'),
                    minWidth: 75,
                    hidden: true,
                    handler: function () {

                        fn.win.el.mask(_S('COMMON', 'APPLYING'));

                        //handle enable
                        AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, { act: 'enable' }),
                            method: 'post',
                            success: function () {
                                fn.accountPanel.items.get('title').items.get('start').hide();
                                fn.accountPanel.items.get('title').items.get('stop').show();
                                fn.win.el.unmask();
                            },
                            failure: function (json, opts) {
                                AS.ARC.util.showMsgWindow({
                                    5022: _S('DATASYNC_DROPBOX', 'FILE_IS_BUSY'),
                                    5026: _S('DATASYNC_DROPBOX', 'FILE_SYSTEM_READ_ONLY'),
                                    5401: _S('DATASYNC_DROPBOX', 'UNKNOW_ERROR'),
                                    5402: _S('DATASYNC_DROPBOX', 'OUT_OF_MEMORY'),
                                }, json, fn.win);
                                fn.win.el.unmask();
                            }
                        });
                    }
                }, {
                    xtype: 'button',
                    itemId: 'stop',
                    minWidth: 75,
                    maxWidth: 150,
                    hidden: true,
                    text: _S('DATASYNC_DROPBOX', 'STOP'),
                    handler: function () {

                        fn.win.el.mask(_S('COMMON', 'APPLYING'));

                        // handle disable
                        AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, { act: 'disable' }),
                            method: 'post',
                            success: function () {
                                fn.accountPanel.items.get('title').items.get('start').show();
                                fn.accountPanel.items.get('title').items.get('stop').hide();
                                fn.win.el.unmask();
                            },
                            failure: function (json, opts) {

                                AS.ARC.util.showMsgWindow({
                                    5022: _S('DATASYNC_DROPBOX', 'FILE_IS_BUSY'),
                                    5026: _S('DATASYNC_DROPBOX', 'FILE_SYSTEM_READ_ONLY'),
                                    5401: _S('DATASYNC_DROPBOX', 'UNKNOW_ERROR'),
                                    5402: _S('DATASYNC_DROPBOX', 'OUT_OF_MEMORY'),
                                }, json, fn.win);

                                fn.win.el.unmask();
                            }
                        });
                    }
                }, {
                    xtype: 'button',
                    itemId: 'settingPanelBtn',
                    text: _S('COMMON', 'SETTING'),
                    minWidth: 75,
                    handler: function () {
                        fn.openSettingWindow();
                    }
                }, {
                    xtype: 'tbfill'
                }, {
                    xtype: 'button',
                    scale: 'medium',
                    iconCls: 'disconnect',
                    listeners: {
                        render: function (btn, eOpts) {
                            Ext.create('Ext.tip.ToolTip', {
                                target: btn.getEl(),
                                html: _S('DATASYNC_DROPBOX', 'UNLINK')
                            });
                        },
                        click: function () {
                            fn.doLogout();
                        }
                    }
                }]
            }, {
                xtype: 'fieldset',
                itemId: 'fs',
                title: _S('DATASYNC_DROPBOX', 'ACCOUNT_INFORMATION'),
                style: 'margin-top: 10px;',
                defaults: {
                    width: 545
                },
                items: [{
                    xtype: 'displayfield',
                    fieldLabel: _S('DATASYNC_DROPBOX', 'NAME'),
                    labelWidth: AS.ARC.config.layout.labelWidthSmall + 10,
                    itemId: 'name',
                    value: AS.ARC.datas.defaultValue
                }, {
                    xtype: 'displayfield',
                    fieldLabel: _S('DATASYNC_DROPBOX', 'LINK'),
                    labelWidth: AS.ARC.config.layout.labelWidthSmall + 10,
                    itemId: 'link',
                    value: AS.ARC.datas.defaultValue
                }, {
                    xtype: 'displayfield',
                    fieldLabel: _S('DATASYNC_DROPBOX', 'STATUS'),
                    labelWidth: AS.ARC.config.layout.labelWidthSmall + 10,
                    itemId: 'status',
                    value: AS.ARC.datas.defaultValue
                }, {
                    xtype: 'container',
                    margin: '15 0 0 0',
                    itemId: 'locationData',
                    width: 600,
                    layout: 'hbox',
                    height: 28,
                    items: [{
                        xtype: 'textfield',
                        itemId: 'location',
                        width: 350,
                        readOnly: true,
                        labelWidth: AS.ARC.config.layout.labelWidthSmall + 10,
                        fieldLabel: _S('DATASYNC_DROPBOX', 'LOCATION'),
                        listeners: {
                            afterrender: function (df) {

                                AS.ARC.ajax({
                                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, { act: 'get_conf' }),
                                    method: 'post',
                                    success: function (json, opts) {
                                        df.setValue(json.conf.local_root);
                                    },
                                    failure: function (json, opts) {
                                    }
                                });
                            }
                        }
                    }, {
                        xtype: 'button',
                        margin: '0 0 0 5',
                        minWidth: 75,
                        text: _S('DATASYNC_DROPBOX', 'OPEN'),
                        handler: function (btn) {

                            AS.ARC.ajax({
                                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, { act: 'get_conf' }),
                                method: 'post',
                                success: function (json, opts) {
                                    var path = json.conf.local_root;
                                    if (path.indexOf('/Home/') === 0) {
                                        path = path.replace('/Home', '/home');
                                    }
                                    AS.ARC.core.openApp('app-fileExplorer', path);
                                },
                                failure: function (json, opts) {

                                }
                            });
                        }
                    }]
                }, {
                    xtype: 'displayfield',
                    fieldLabel: _S('DATASYNC_DROPBOX', 'QUOTA'),
                    labelWidth: AS.ARC.config.layout.labelWidthSmall + 10,
                    itemId: 'quota',
                    value: AS.ARC.datas.defaultValue
                },
                    capacityBar,
                    sizeBar, {
                    xtype: 'fileList',
                    itemId: 'error-file-list',
                    fn: fn
                }]
            }],
            loadData: function () {
                fn.win.el.unmask();
                fn.getStatus();
                fn.getUserInfo();
            }
        });

        return fn.accountPanel;
    },
    openSettingWindow: function () {
        var fn = this;
        var radioName = 'syncOption2';
        var applyValidate = function() {
            var root = serverTree.getRootNode();
            if(root.findChild('checked', true, true) && sizeLimit.isValid() && uploadLimit.isValid() && downloadLimit.isValid()) {
                applyBtn.enable();
            } else {
                applyBtn.disable();
            }
        }
        var applyBtn = Ext.create('Ext.button.Button', {
            xtype: 'button',
            itemId: 'Apply',
            text: _S('COMMON', 'APPLY'),
            disabled: true,
            handler: function(btn) {
                settingWindow.el.mask(_S('COMMON', 'APPLYING'));
                btn.disable();
                //filter setting
                var filterPanel = settingWindow.items.get('settingTabPanel').items.get('filterTab').items.get('filter-panel');
                var filterSetPanel = filterPanel.items.get('filter-set').items.get('filter-set-panel');
                var documentCt = filterSetPanel.items.get('documentCt').items.get('ct2').query('checkbox');
                var videoCt = filterSetPanel.items.get('videoCt').items.get('ct2').query('checkbox');
                var musicCt = filterSetPanel.items.get('musicCt').items.get('ct2').query('checkbox');
                var photoCt = filterSetPanel.items.get('photoCt').items.get('ct2').query('checkbox');
                var applicationCt = filterSetPanel.items.get('applicationCt').items.get('ct2').query('checkbox');
                var customCt = filterSetPanel.items.get('customCt').items.get('ct2').query('checkbox');
                var sizeLimit = filterPanel.items.get('size_limit').getValue();
                var filterExtName = [];
                var filterName = [];
                var userDefExtName = [];
                //folder setting
                var location = fn.accountPanel.items.get('fs').items.get('locationData').items.get('location');
                var path = settingWindow.items.get('settingTabPanel').items.get('serverTreeTab').items.get('path-ct').down('[itemId=path_df]').getValue();
                var physicalPath = '\/volume1' + path;
                var filterPath = [];
                //config setting
                var configPanel = settingWindow.items.get('settingTabPanel').items.get('configTab').items.get('configPanel');
                var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit').getValue();
                var downloadLimit = configPanel.items.get('fs1').items.get('download_limit').getValue();
                var radiogroup = configPanel.items.get('fs2').query('radiogroup')[0];
                var enableUpload = true;
                var enableDownload = true;
                var deleteExtra = true;
                if(radiogroup.getValue()[radioName] == 'DROPBOX_TO_NAS') {
                    enableUpload = false;
                    deleteExtra = (configPanel.items.get('fs2').items.get('delete_extra').getValue());
                } else if(radiogroup.getValue()[radioName] == 'NAS_TO_DROPBOX') {
                    enableDownload = false;
                    deleteExtra = (configPanel.items.get('fs2').items.get('delete_extra').getValue());
                }
                //folder info
                serverTree.getRootNode().cascadeBy(function(n) {
                    if(!n.data.root && !n.data.checked) {
                        filterPath.push(n.data.path);
                    }
                });
                //filter info
                for (var i = 0; i < documentCt.length; i ++) {
                    if (documentCt[i].getValue() === true) {
                        filterExtName.push(documentCt[i].boxLabel);
                    }
                }
                for (var i = 0; i < videoCt.length; i ++) {
                    if (videoCt[i].getValue() === true) {
                        filterExtName.push(videoCt[i].boxLabel);
                    }
                }
                for (var i = 0; i < musicCt.length; i ++) {
                    if (musicCt[i].getValue() === true) {
                        filterExtName.push(musicCt[i].boxLabel);
                    }
                }
                for (var i = 0; i < photoCt.length; i ++) {
                    if (photoCt[i].getValue() === true) {
                        filterExtName.push(photoCt[i].boxLabel);
                    }
                }
                for (var i = 0; i < applicationCt.length; i ++) {
                    if (applicationCt[i].getValue() === true) {
                        filterExtName.push(applicationCt[i].boxLabel);
                    }
                }
                for (var i = 0; i < customCt.length; i ++) {
                    userDefExtName.push(customCt[i].boxLabel);
                    if (customCt[i].getValue() === true) {
                        filterExtName.push(customCt[i].boxLabel);
                    }
                }
						
                var conf = {
                    'local_root': path,
                    'local_root_path': physicalPath,
                    'size_limit': sizeLimit,
                    'upload_limit': uploadLimit,
                    'download_limit': downloadLimit,
                    'upload': enableUpload,
                    'download': enableDownload,
                    'delete_remote': deleteExtra,
                    'delete_local': deleteExtra,
                    'filter_extname': filterExtName,
                    'user_def_extname': userDefExtName,
                    'filter_name': [],
                    'filter_path': filterPath
                };
                
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'set_conf', conf: JSON.stringify(conf)}),
                    method: 'post',
                    success: function (json, opts) {
                        settingWindow.el.unmask();
                        btn.enable();
                        location.setValue(path);
                        settingWindow.close();
                    },
                    failure: function (json, opts) {
                        settingWindow.el.unmask();
                        btn.enable();
                        settingWindow.close();
                    }
                });

            }
        });
        //tab server tree
        var getFileBrowse = function() {
            var savePath = Ext.create('Ext.form.field.Text', {
                itemId: 'path_df',
                dirtyPath: false,
                width: 390,
                value: AS.ARC.datas.defaultValue,
                readOnly: true,
                allowBlank: false,
                blankText: _S('MESSAGE', 'HF00011'),
            });

            var browseBtn = Ext.create(Ext.button.Button, {
                itemId: 'browse',
                margin: '0 0 0 10',
                text: _S('COMMON', 'BROWSE'),
                handler: function () {
                    var fs = Ext.create('AS.ARC.treeSelector', {
                        hideReadOnly: true,
                        parentWin: settingWindow,
                        objectPath: savePath,
                        isShowHome: true,
                        hideUserName: true,
                        unselectableFolder: [
                            '/home/' + AS.ARC.user.account,
                            '/home/' + AS.ARC.user.account + '/.datasync-dropbox'
                        ]
                    });
                    fs.win.show();
                }
            });

            var ct = Ext.create(Ext.form.FieldSet, {
                itemId: 'path-ct',
                title: _S('DATASYNC_DROPBOX', 'PLEASE_SELECT_LOCAL_FOLDER'),
                padding: 0,
                items: [{
                    xtype: 'container',
                    layout: 'hbox',
                    margin: 0,
                    items: [
                        savePath,
                        browseBtn
                    ]
                }]
            });
            return ct;
        };
        var serverTree = fn.getServerTree(applyBtn);
        serverTree.on('itemclick', applyValidate);
        var serverTreeTab = Ext.create(AS.ARC.panelTop10, {
            itemId: 'serverTreeTab',
            title: _S('DATASYNC_DROPBOX', 'FOLDER_SETTINGS'),
            items: [
                getFileBrowse(),
                {
                    xtype: 'fieldset',
                    itemId: 'fs',
                    title: _S('DATASYNC_DROPBOX', 'DO_NOT_SYNC_FOLDER'),
                    style: 'margin-top: 5px;',
                    padding: 0,
                    border: false,
                    //width: 533,
                    height: 300,
                    items: [
                        serverTree
                    ]
                },
                {
                    xtype: 'displayfield',
                    itemId: 'path',
                    hidden: true
                }
            ]
        });
        //tab filter
        var filterPanel = fn.getFilterPanel(325);
        var sizeLimit =  filterPanel.items.get('size_limit');
        sizeLimit.on('validitychange', applyValidate);
        var filterTab = Ext.create(AS.ARC.panelTop10, {
            itemId: 'filterTab',
            title: _S('DATASYNC_DROPBOX', 'FILE_FILTER'),
            items: [
                filterPanel
            ]
        });
        //tab config
        var configPanel = fn.getConfigPanel(radioName);
        var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit');
        uploadLimit.on('validitychange', applyValidate);
        var downloadLimit = configPanel.items.get('fs1').items.get('download_limit');
        downloadLimit.on('validitychange', applyValidate);
        var configTab = Ext.create(AS.ARC.panelTop10, {
            itemId: 'configTab',
            title: _S('COMMON', 'ADVANCED_SETTINGS'),
            items: [
                configPanel
            ]
        });
        var tabPanel = Ext.create(Ext.tab.Panel, {
            itemId: 'settingTabPanel',
            cls: 'as-tab-panel',
            width: 575,
            height: 600,
            items: [
                serverTreeTab,
                filterTab,
                configTab
            ],
            listeners: {
                afterrender: function(me) {
                    me.addCls('datasyncDropbox-tab');
                }
            }
        });

        var settingWindow = Ext.create('AS.ARC.subWindow', {
            title: _S('COMMON', 'SETTINGS'),
            width: 575,
            height: 600,
            parentWin: fn.win,
            activeItem: 0,
            items: [
                tabPanel
            ],
            fbar: [
                applyBtn,
                {
                    xtype: 'button',
                    itemId: 'cancel',
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        settingWindow.close();
                    }
                }
            ],
            //get data to show on setting window
            listeners: {
                afterrender: function() {
                    //folder setting
                    var path = settingWindow.items.get('settingTabPanel').items.get('serverTreeTab').items.get('path-ct').down('[itemId=path_df]');
                    //filter setting
                    var filterPanel = settingWindow.items.get('settingTabPanel').items.get('filterTab').items.get('filter-panel');
                    var filterSetPanel = filterPanel.items.get('filter-set').items.get('filter-set-panel');
                    var documentCt = filterSetPanel.items.get('documentCt');
                    var videoCt = filterSetPanel.items.get('videoCt');
                    var musicCt = filterSetPanel.items.get('musicCt');
                    var photoCt = filterSetPanel.items.get('photoCt');
                    var applicationCt = filterSetPanel.items.get('applicationCt');
                    var customCt = filterSetPanel.items.get('customCt');
                    var documentCbs = filterSetPanel.items.get('documentCt').items.get('ct2').query('checkbox');
                    var videoCbs = filterSetPanel.items.get('videoCt').items.get('ct2').query('checkbox');
                    var musicCbs = filterSetPanel.items.get('musicCt').items.get('ct2').query('checkbox');
                    var photoCbs = filterSetPanel.items.get('photoCt').items.get('ct2').query('checkbox');
                    var applicationCbs = filterSetPanel.items.get('applicationCt').items.get('ct2').query('checkbox');
                    var customCbs = filterSetPanel.items.get('customCt').items.get('ct2').query('checkbox');
                    var sizeLimit = filterPanel.items.get('size_limit');
                    //config setting
                    var configPanel = settingWindow.items.get('settingTabPanel').items.get('configTab').items.get('configPanel');
                    var uploadLimit = configPanel.items.get('fs1').items.get('upload_limit');
                    var downloadLimit = configPanel.items.get('fs1').items.get('download_limit');
                    var deleteExtra = configPanel.items.get('fs2').items.get('delete_extra');
                    var radiogroup = configPanel.items.get('fs2').query('radiogroup')[0];
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get_conf'}),
                        method: 'post',
                        success: function (json, opts) {
                            settingWindow.el.unmask();
                            //folder setting
                            path.setValue(json.conf.local_root);
                            //config setting
                            if (json.conf.user_def_extname.length != 0) {
                                for (var i = 0; i < json.conf.user_def_extname.length; i ++) {
                                    filterPanel.items.get('filter-set').addCustomCheckbox(json.conf.user_def_extname[i], json.conf.filter_extname);
                                }
                            }
                            
                            uploadLimit.setValue(json.conf.upload_limit);
                            downloadLimit.setValue(json.conf.download_limit);
                            var o = {};
                            if (json.conf.upload && json.conf.download) {
                                o[radioName] = 'BOTH';
                                radiogroup.setValue(o);
                            } else if (json.conf.download && !json.conf.upload) {
                                o[radioName] = 'DROPBOX_TO_NAS';
                                radiogroup.setValue(o);
                                deleteExtra
                                    .setValue(json.conf.delete_local)
                                    .show();
                            } else if (!json.conf.download && json.conf.upload) {
                                o[radioName] = 'NAS_TO_DROPBOX';
                                radiogroup.setValue(o);
                                deleteExtra
                                    .setValue(json.conf.delete_remote)
                                    .show();
                            }
                            //filter setting
                            if (json.conf.filter_extname.length != 0) {
                                for(var i = 0; i < json.conf.filter_extname.length; i ++) {
                                    for (var j = 0; j < documentCbs.length; j ++) {

                                        if (json.conf.filter_extname[i] === documentCbs[j].boxLabel) {
                                            documentCbs[j].setValue(true);
                                        }
                                    }
                                    for (var j = 0; j < videoCbs.length; j ++) {
                                        if (json.conf.filter_extname[i] === videoCbs[j].boxLabel) {
                                            videoCbs[j].setValue(true);
                                        }
                                    }
                                    for (var j = 0; j < musicCbs.length; j ++) {
                                        if (json.conf.filter_extname[i] === musicCbs[j].boxLabel) {
                                            musicCbs[j].setValue(true);
                                        }
                                    }
                                    for (var j = 0; j < photoCbs.length; j ++) {
                                        if (json.conf.filter_extname[i] === photoCbs[j].boxLabel) {
                                            photoCbs[j].setValue(true);
                                        }
                                    }
                                    for (var j = 0; j < applicationCbs.length; j ++) {
                                        if (json.conf.filter_extname[i] === applicationCbs[j].boxLabel) {
                                            applicationCbs[j].setValue(true);
                                        }
                                    }
                                    for (var j = 0; j < customCbs.length; j ++) {
                                        if (json.conf.filter_extname[i] === customCbs[j].boxLabel) {
                                            customCbs[j].setValue(true);
                                        }
                                    }
                                }
                            }
                            sizeLimit.setValue(json.conf.size_limit);
                        },
                        failure: function (json, opts) {
                        }
                    });
                }
            }
        });

        settingWindow.show();

    },
    //Settings util
    getServerTree: function (applyBtn) {
        var fn = this;
        var serverTreeStore = Ext.create('Ext.data.TreeStore', {
            proxy: {
                type: 'ajax',
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'list_dir'}),
                reader: {
                    type: 'json',
                    root: 'children'
                },
                extraParams:{
                    id: '',
                    path: ''
                }
            },
            fields: [
                {name: 'text'},
                {name: 'path'},
                {name: 'dirLoaded'}
            ],
            listeners: {
                load: function() {
                    tree.el.unmask();
                }
            }
        });

        var tree = new Ext.tree.TreePanel({
            //width: 500,
            height: 250,
            itemId: 'servertree',
            rootVisible: false,
            useArrows : true,
            autoScroll: true,
            resizable: false,
            store: serverTreeStore,
            displayField: 'text',
            animate: false,
            listeners : {
                checkchange: function(node, checked, options) {
                    //子節點打勾 所有父節點也都要打勾
                    if(checked) {
                        node.bubble(function(n) {
                            n.set('checked', checked);
                        });
                    }
                    //子節點若還沒載入 則只做展開的動作 處理交給beforeitemexpand和itemexpand
                    if(!node.isLoaded()) {
                        node.expand();
                    } else {
                        node.expand(false, function() {
                            node.cascadeBy(function(n) {
                                n.set('checked', checked);
                                n.set('cls', '');
                            });
                        });
                    }
                    //處理半勾狀態
                    //子節點有一個以上沒打勾 所有父節點都要是半勾狀態
                    node.set('cls', '');
                    var parent = node.parentNode;
                    if(parent) {
                        var halfStatus = false;
                        parent.bubble(function(n) {
                            if(!halfStatus) {
                                n.eachChild(function(child) {
                                    if(!child.data.checked || child.data.cls === 'x-tree-checked-down') {
                                        return (halfStatus = true);
                                    }
                                });
                            }
                            if(halfStatus) {
                                n.set('cls', 'x-tree-checked-down');
                            } else {
                                n.set('cls', '');
                            }
                        });
                    }
                },
                beforeitemexpand: function(node) {
                    if(!node.isLoaded()) {
                        var root = tree.getStore().getProxy();
                        root.extraParams.id = node.data.id;
                        root.extraParams.path = node.data.path;
                    } else {
                        node.data.dirLoaded = true;
                    }
                },
                itemexpand: function(node) {
                    if(!node.data.dirLoaded) {
                        node.data.dirLoaded = true;
                        node.cascadeBy(function(n){
                            n.data.checked = node.data.checked;
                        });
                    }
                },
                load: function() {
                    if(applyBtn) {
                        applyBtn.enable();
                    }
                }
            }
        });

        return tree;
    },
    getFilterPanel: function(height) {
        var fn = this;
        var height = height || 350;
        var filterSet = Ext.create('AS.ARC.filterSet', {
            imageUrl: AS.ARC.util.getUserAppsPath() + 'datasyncDropbox/images/',
            height: height,
            addRuleTips: [
                _S('DATASYNC_DROPBOX', 'FILTER_RULES_1'),
                _S('DATASYNC_DROPBOX', 'FILTER_RULES_2'),
                _S('DATASYNC_DROPBOX', 'FILTER_RULES_3'),
                _S('DATASYNC_DROPBOX', 'FILTER_RULES_4'),
                _S('DATASYNC_DROPBOX', 'FILTER_RULES_5')
            ]
        });
        var filterPanel = Ext.create(Ext.panel.Panel, {
            itemId: 'filter-panel',
            height: 450,
            border: false,
            items: [
                {
                    xtype: 'displayfield',
                    margin: '5 0 0 0',
                    width: 533,
                    value: _S('DATASYNC_DROPBOX', 'FILTER_TEXT')
                },
                {
                    xtype: 'numberfield',
                    itemId: 'size_limit',
                    margin: '15 0',
                    labelWidth: 150,
                    fieldLabel: _S('DATASYNC_DROPBOX', 'SIZE_LIMIT'),
                    minValue: 0,
                    maxValue: 10239,
                    allowDecimals: false,
                    allowBlank: false,
                    blankText: _S('MESSAGE', 'HF00011'),
                    msgTarget: AS.ARC.config.msgTarget,
                    value: 0
                },
                filterSet
            ]
        });

        return filterPanel;
    },
    getConfigPanel: function(radioName) {
        var fn = this;
        
        radioName = radioName ? radioName : 'syncOption';
        var configPanel = Ext.create(Ext.panel.Panel, {
            itemId: 'configPanel',
            border: false,
            items: [
                {
                    xtype: 'fieldset',
                    itemId: 'fs1',
                    title: _S('DATASYNC_DROPBOX', 'TRAFFIC_CONTROL'),
                    //height: 100,
                    items: [
                        {
                            xtype: 'numberfield',
                            itemId: 'upload_limit',
                            labelWidth: 300,
                            fieldLabel: _S('DATASYNC_DROPBOX', 'UPLOAD_LIMIT'),
                            minValue: 0,
                            maxValue: 999999,
                            allowDecimals: false,
                            allowBlank: false,
                            blankText: _S('MESSAGE', 'HF00011'),
                            msgTarget: AS.ARC.config.msgTarget,
                            value: 0
                        },
                        {
                            xtype: 'numberfield',
                            itemId: 'download_limit',
                            labelWidth: 300,
                            fieldLabel: _S('DATASYNC_DROPBOX', 'DOWNLOAD_LIMIT'),
                            minValue: 0,
                            maxValue: 999999,
                            allowDecimals: false,
                            allowBlank: false,
                            blankText: _S('MESSAGE', 'HF00011'),
                            msgTarget: AS.ARC.config.msgTarget,
                            value: 0
                        }
                    ]
                },
                {
                    xtype: 'fieldset',
                    itemId: 'fs2',
                    title: _S('COMMON', 'SYNCHRONIZATION_SETTINGS'),
                    items: [
                        {
                            xtype: 'radiogroup',
                            columns: 1,
                            margin: 0,
                            items: [
                                {
                                    xtype: 'radio',
                                    boxLabel: _S('DATASYNC_DROPBOX', 'SYNCHRONIZITION'),
                                    name: radioName,
                                    inputValue: 'BOTH',
                                    checked: true
                                },
                                {
                                    xtype: 'radio',
                                    boxLabel: _S('DATASYNC_DROPBOX', 'DATASYNC_DROPBOX_TO_NAS'),
                                    name: radioName,
                                    inputValue: 'DROPBOX_TO_NAS',
                                },
                                {
                                    xtype: 'radio',
                                    boxLabel: _S('DATASYNC_DROPBOX', 'NAS_TO_DATASYNC_DROPBOX'),
                                    name: radioName,//'syncOption',
                                    inputValue: 'NAS_TO_DROPBOX',
                                }
                            ],
                            listeners: {
                                change: function(radiogroup, newValue) {
                                    if(newValue[radioName] == 'BOTH') {
                                        configPanel.items.get('fs2').items.get('delete_extra').hide();
                                    } else {
                                        configPanel.items.get('fs2').items.get('delete_extra').show();
                                    }
                                }
                            }
                        },
                        {
                            xtype: 'checkbox',
                            itemId: 'delete_extra',
                            hidden: true,
                            //width: configPanel.items.get('fs2').el.getWidth() - 20,
                            style: 'margin-left: 20px',
                            boxLabel: _S('DATASYNC_DROPBOX', 'KEEP_EXTRA'),
                            checked: true
                        }
                    ]
                },
                fn.getUploadNote()
            ],
            listeners: {
                afterrender: function(p) {
                    var check = p.items.get('fs2').items.get('delete_extra');
                    var width = p.items.get('fs2').el.getWidth() - 20;
                    check.setWidth(width);
                }
            }
        });

        return configPanel;
    },
    //Login
    doLogout: function () {
        var fn = this;
		fn.win.el.mask(_S('COMMON', 'LOADING'));
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'logout'}),
            method: 'post',
            success: function (json, opts) {
                fn.connCounter.clear();
                fn.mainPanel.getLayout().setActiveItem(0);
                clearTimeout(fn.userInfo);
                clearTimeout(fn.status);
                fn.checkLogin(true);
                fn.win.el.unmask();
            },
            failure: function (json, opts) {
                /*AS.ARC.util.showMsgWindow({
                    5022: _S('DATASYNC_DROPBOX', 'FILE_IS_BUSY'),
                    5026: _S('DATASYNC_DROPBOX', 'FILE_SYSTEM_READ_ONLY'),
                    5401: _S('DATASYNC_DROPBOX', 'UNKNOW_ERROR'),
                    5402: _S('DATASYNC_DROPBOX', 'OUT_OF_MEMORY'),
                }, json, fn.win);*/
            }
        });
    },
    doLogin: function () {
        var fn = this;
        
        fn.win.el.mask(_S('COMMON', 'LOADING'));
        //btn.disable();
        var url = fn.address + escape('&adm_url=' + window.location.href.split('portal').shift());
        var w = window.open(url, "newwindow", "height=480, width=480,toolbar=no,menubar=no, scrollbars=no, resizable=no, location=no, status=no");
        var winCloseDetector = setInterval(function () {
            try {
                if (w == null || w.closed) {
                    clearInterval(winCloseDetector);
                }
            }
            catch (e) {
            }
        }, 1000);

    },
    checkLogin: function (tryLogin) {
        var fn = this;
        //Login / Settle
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'check_login'}),
            method: 'post',
            success: function (json, opts) {
                if (!json.login) {
                    if (tryLogin) {
                        fn.address = json.url;
                        fn.loginPanel.items.get('signinaddress').enable();
                        fn.win.el.unmask();

                        fn.loginID = setTimeout(function () {
                            fn.checkLogin(true);
                        }, 3000);
                    } else {
                        fn.doLogout();
                    }
                } else {
                    //have linked and logged in
                    if (tryLogin) {
                        AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'check_set'}),
                            method: 'post',
                            success: function (json, opts) {
                                if (!json.settle) {
                                    //go to SettingPanel
                                    fn.settingPanel.loadData();
                                    fn.mainPanel.getLayout().setActiveItem(1);
                                    fn.win.el.unmask();
                                } else {
                                    //go to AccountPanel
                                    fn.accountPanel.loadData();
                                    fn.mainPanel.getLayout().setActiveItem(2);
                                }
                            },
                            failure: function (json, opts) {}
                        });
                    }
                }
            },
            failure: function (json, opts) {
                AS.ARC.util.showMsgWindow({
                    5601: _S('ERROR', 'CONNECTION_FAIL')
                }, json, fn.win);
            }
        });
    },

    getMainPanel: function () {
        var fn = this;

        fn.mainPanel = Ext.create('Ext.panel.Panel', {
            cls: 'as-page-panel',
            region: 'center',
            height: 500,
            width: 600,
            border: false,
            layout: {
                type: 'card'
            },
            activeItem: 0,
            items: [
                fn.getLoginPanel(),
                fn.getSettingPanel(),
                fn.getAccountPanel()
            ],
            loadData: function () {
                fn.checkLogin(true);
            }
        });
        return fn.mainPanel;
    }
});

/**
 * @class AS.ARC.apps.datasyncDropbox.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.datasyncDropbox.main', {
    extend: 'AS.ARC._appBase',

    appTag: 'datasyncDropbox',
    appMaxNum: 1,
    appOpenNum: 0,
    appIsReady: true,
    appWins: [],

    createWindow: function () {
        var desktop = this.core.getDesktop(),
            app = this;

        if ((this.appOpenNum === this.appMaxNum) || !this.appIsReady) {
            this.appWins[0].show();
            return;
        }

        // Set appIsReady to false
        this.appIsReady = false;

        var newDropbox = new AS.ARC.apps.datasyncDropbox.core({
            app: this,
            desktop: desktop,
            id: this.id + '-' + Ext.id()
        });

        newDropbox.win.on('render', function () {
            app.appOpenNum++;
            app.appIsReady = true;
        });

        newDropbox.win.on('beforeclose', function () {
            app.appOpenNum--;
            app.appIsReady = true;
            app.appWins.pop();
            clearTimeout(newDropbox.loginID);
            clearTimeout(newDropbox.userInfo);
            clearTimeout(newDropbox.status);
        });
        newDropbox.win.show();

        // Get all wins for handling
        this.appWins.push(newDropbox.win);
    },
    onClose: function () {

    }
});
